@@ -162,6 +162,33 @@ public:
 
 NAMESPACE_END(detail)
 
+// *WARNING*: nanobind regularly receives requests from users who run it
+// through Clang-Tidy, or who compile with increased warnings levels, like
+//
+//     -Wcast-qual, -Wsign-conversion, etc.
+//
+// (i.e., beyond -Wall -Wextra and /W4 that are currently already used)
+//
+// Their next step is to open a big pull request needed to silence all of
+// the resulting messages.  This comment is strategically placed here
+// because the (PyObject *) casts below cast away the const qualifier and
+// will almost certainly be flagged in this process.
+//
+// My policy on this is as follows: I am always happy to fix issues in the
+// codebase.  However, many of the resulting change requests are in the
+// "ritual purification" category: things that cause churn, decrease
+// readability, and which don't fix actual problems.  It's a never-ending
+// cycle because each new revision of such tooling adds further warnings
+// and purification rites.
+//
+// So just to be clear: I do not wish to pepper this codebase with
+// "const_cast" and #pragmas/comments to avoid warnings in external
+// tooling just so those users can have a "silent" build.  I don't think it
+// is reasonable for them to impose their own style on this project.
+//
+// As a workaround it is likely possible to restrict the scope of style
+// checks to particular C++ namespaces or source code locations.
+
 class handle : public detail::api<handle> {
     friend class python_error;
     friend struct detail::str_attr;
@@ -348,6 +375,12 @@ class capsule : public object {
     const char *name() const { return PyCapsule_GetName(m_ptr); }
 
     void *data() const { return PyCapsule_GetPointer(m_ptr, name()); }
+    void *data(const char *name) const {
+        void *p = PyCapsule_GetPointer(m_ptr, name);
+        if (!p && PyErr_Occurred())
+            raise_python_error();
+        return p;
+    }
 };
 
 class bool_ : public object {
@@ -421,7 +454,7 @@ class str : public object {
     explicit str(const char *s, size_t n)
         : object(detail::str_from_cstr_and_size(s, n), detail::steal_t{}) { }
 
-    template <typename... Args> str format(Args&&... args);
+    template <typename... Args> str format(Args&&... args) const;
 
     const char *c_str() const { return PyUnicode_AsUTF8AndSize(m_ptr, nullptr); }
 };
@@ -445,16 +478,17 @@ class bytes : public object {
     size_t size() const { return (size_t) PyBytes_Size(m_ptr); }
 };
 
+NAMESPACE_BEGIN(literals)
+inline str operator""_s(const char *s, size_t n) {
+    return str(s, n);
+}
+NAMESPACE_END(literals)
+
 class bytearray : public object {
     NB_OBJECT(bytearray, object, "bytearray", PyByteArray_Check)
 
-#if PY_VERSION_HEX >= 0x03090000
     bytearray()
         : object(PyObject_CallNoArgs((PyObject *)&PyByteArray_Type), detail::steal_t{}) { }
-#else
-    bytearray()
-        : object(PyObject_CallObject((PyObject *)&PyByteArray_Type, NULL), detail::steal_t{}) { }
-#endif
 
     explicit bytearray(handle h)
         : object(detail::bytearray_from_obj(h.ptr()), detail::steal_t{}) { }
@@ -488,6 +522,7 @@ class tuple : public object {
     detail::fast_iterator begin() const;
     detail::fast_iterator end() const;
 #endif
+    bool empty() const { return size() == 0; }
 };
 
 class type_object : public object {
@@ -531,6 +566,7 @@ class list : public object {
     detail::fast_iterator begin() const;
     detail::fast_iterator end() const;
 #endif
+    bool empty() const { return size() == 0; }
 };
 
 class dict : public object {
@@ -542,12 +578,25 @@ class dict : public object {
     list keys() const { return steal<list>(detail::obj_op_1(m_ptr, PyDict_Keys)); }
     list values() const { return steal<list>(detail::obj_op_1(m_ptr, PyDict_Values)); }
     list items() const { return steal<list>(detail::obj_op_1(m_ptr, PyDict_Items)); }
+    object get(handle key, handle def) const {
+        PyObject *o = PyDict_GetItem(m_ptr, key.ptr());
+        if (!o)
+            o = def.ptr();
+        return borrow(o);
+    }
+    object get(const char *key, handle def) const {
+        PyObject *o = PyDict_GetItemString(m_ptr, key);
+        if (!o)
+            o = def.ptr();
+        return borrow(o);
+    }
     template <typename T> bool contains(T&& key) const;
     void clear() { PyDict_Clear(m_ptr); }
     void update(handle h) {
         if (PyDict_Update(m_ptr, h.ptr()))
             raise_python_error();
     }
+    bool empty() const { return size() == 0; }
 };
 
 class set : public object {
@@ -563,14 +612,25 @@ class set : public object {
             raise_python_error();
     }
     template <typename T> bool discard(T &&value);
+    bool empty() const { return size() == 0; }
+};
+
+class frozenset : public object {
+    NB_OBJECT(frozenset, object, "frozenset", PyFrozenSet_Check)
+    frozenset() : object(PyFrozenSet_New(nullptr), detail::steal_t()) { }
+    explicit frozenset(handle h)
+        : object(detail::frozenset_from_obj(h.ptr()), detail::steal_t{}) { }
+    size_t size() const { return (size_t) NB_SET_GET_SIZE(m_ptr); }
+    template <typename T> bool contains(T&& key) const;
+    bool empty() const { return size() == 0; }
 };
 
 class sequence : public object {
-    NB_OBJECT_DEFAULT(sequence, object, NB_TYPING_SEQUENCE, PySequence_Check)
+    NB_OBJECT_DEFAULT(sequence, object, "collections.abc.Sequence", PySequence_Check)
 };
 
 class mapping : public object {
-    NB_OBJECT_DEFAULT(mapping, object, NB_TYPING_MAPPING, PyMapping_Check)
+    NB_OBJECT_DEFAULT(mapping, object, "collections.abc.Mapping", PyMapping_Check)
     list keys() const { return steal<list>(detail::obj_op_1(m_ptr, PyMapping_Keys)); }
     list values() const { return steal<list>(detail::obj_op_1(m_ptr, PyMapping_Values)); }
     list items() const { return steal<list>(detail::obj_op_1(m_ptr, PyMapping_Items)); }
@@ -592,7 +652,7 @@ public:
     using reference = const handle;
     using pointer = const handle *;
 
-    NB_OBJECT_DEFAULT(iterator, object, NB_TYPING_ITERATOR, PyIter_Check)
+    NB_OBJECT_DEFAULT(iterator, object, "collections.abc.Iterator", PyIter_Check)
 
     iterator& operator++() {
         m_value = steal(detail::obj_iter_next(m_ptr));
@@ -624,7 +684,7 @@ private:
 
 class iterable : public object {
 public:
-    NB_OBJECT_DEFAULT(iterable, object, NB_TYPING_ITERABLE, detail::iterable_check)
+    NB_OBJECT_DEFAULT(iterable, object, "collections.abc.Iterable", detail::iterable_check)
 };
 
 /// Retrieve the Python type object associated with a C++ class
@@ -693,6 +753,12 @@ public:
     }
 };
 
+class memoryview : public object {
+    NB_OBJECT(memoryview, object, "memoryview", PyMemoryView_Check)
+    explicit memoryview(handle h)
+        : object(detail::memoryview_from_obj(h.ptr()), detail::steal_t{}) { }
+};
+
 class ellipsis : public object {
     static bool is_ellipsis(PyObject *obj) { return obj == Py_Ellipsis; }
 
@@ -711,7 +777,7 @@ public:
 
 class callable : public object {
 public:
-    NB_OBJECT(callable, object, NB_TYPING_CALLABLE, PyCallable_Check)
+    NB_OBJECT(callable, object, "collections.abc.Callable", PyCallable_Check)
     using object::object;
 };
 
@@ -744,9 +810,18 @@ public:
     static bool check_(handle h) { return isinstance<T>(h); }
 };
 
+struct fallback : public handle {
+public:
+    static constexpr auto Name = detail::const_name("object");
+
+    using handle::handle;
+    using handle::operator=;
+    fallback(const handle &h) : handle(h) { }
+};
+
 template <typename T> class type_object_t : public type_object {
 public:
-    static constexpr auto Name = detail::const_name(NB_TYPING_TYPE "[") +
+    static constexpr auto Name = detail::const_name("type[") +
                                  detail::make_caster<T>::Name +
                                  detail::const_name("]");
 
